<?php
require_once($INC . "/util.inc.php");
require_once($INC . "/log.inc.php");
require_once($INC . "/sql.inc.php");
require_once($INC . "/sess.inc.php");
require_once($INC . "/rights.inc.php");

$rg_user_rights = array(
	"C" => "Create repositories",
	"U" => "Add users"
);

rg_rights_register("user", $rg_user_rights);

function rg_user_set_error($str)
{
	global $_rg_user_error;

	$_rg_user_error = $str;
}

function rg_user_error()
{
	global $_rg_user_error;
	return $_rg_user_error;
}

/*
 * Returns the path to user home based on name
 */
function rg_user_name2path($rr)
{
	global $rg_repos;

	$len = strlen($rr['user']);
	$v = $rr['user'];
	if ($len == 1)
		$v .= "_";

	if (empty($rr['prefix']))
		$type = "org";
	else
		$type = "user";

	return $rg_repos . "/" . $type . "s/"
		. $v[0] . "/" . $v[1] . "/" . $rr['user'];
}

/*
 * Returns the URL to a user home based on $ui
 */
function rg_user_url($ui)
{
	$prefix = "";
	if ($ui['organization'] == 0)
		$prefix = "/user";

	return $prefix . "/" . $ui['username'];
}

/*
 * Computes password hash
 */
function rg_user_pass($salt, $pass)
{
	return sha1($salt . "===" . $pass);
}

/*
 * Validates a password
 */
function rg_user_pass_ok($pass)
{
	if (strlen($pass) < 5) {
		rg_user_set_error("password is too short (less than 5 chars)");
		return FALSE;
	}

	return TRUE;
}

/*
 * Returns true if the user is ok
 */
function rg_user_ok($user)
{
	global $rg_user_allow;
	global $rg_user_min_len;
	global $rg_user_max_len;

	if (rg_chars_allow($user, $rg_user_allow) !== TRUE) {
		rg_user_set_error("invalid user name (invalid chars [$user] [$rg_user_allow])");
		return FALSE;
	}

	if (strlen($user) < $rg_user_min_len) {
		rg_user_set_error("user name too short (shorter than $rg_user_min_len)");
		return FALSE;
	}

	if (strlen($user) > $rg_user_max_len) {
		rg_user_set_error("user name too long (longer than $rg_user_max_len)");
		return FALSE;
	}

	return TRUE;
}

/*
 * Add a user
 * If uid > 0 - edit, else, add
 */
function rg_user_edit($db, $d)
{
	global $rg_user_info_cache; // TODO: what we do with this?

	rg_log("user_edit: data: " . rg_array2string($d));

	// DEBUG
	if (!isset($d['username']))
		rg_internal_error("username not passed");

	if (rg_user_ok($d['username']) !== TRUE)
		return FALSE;

	$now = time();
	$e_username = rg_sql_escape($db, $d['username']);
	$e_realname = rg_sql_escape($db, $d['realname']);
	$e_salt = rg_id(40);
	$e_pass = rg_user_pass($e_salt, $d['pass']);
	$e_email = rg_sql_escape($db, $d['email']);
	$e_rights = rg_sql_escape($db, $d['rights']);
	$e_is_admin = $d['is_admin'];
	$e_disk_quota_mb = $d['disk_quota_mb'];
	$e_session_time = $d['session_time'];
	$e_confirm_token = isset($d['confirm_token']) ? $d['confirm_token'] : "";

	if (empty($d['confirm_token'])) {
		// no need to confirm account
		$e_confirmed = $now;
	} else {
		$e_confirmed = 0;
	}

	if ($d['uid'] == 0) { // add
		if (rg_user_pass_ok($d['pass']) !== TRUE)
			return FALSE;

		$sql = "INSERT INTO users (username, realname, salt, pass"
			. ", email, itime"
			. ", is_admin, disk_quota_mb, rights, session_time"
			. ", confirmed, confirm_token)"
			. " VALUES ('$e_username', '$e_realname', '$e_salt', '$e_pass'"
			. ", '$e_email', $now, $e_is_admin, $e_disk_quota_mb"
			. ", '$e_rights', $e_session_time"
			. ", $e_confirmed, '$e_confirm_token')";
	} else { // edit
		$salt_pass_add = "";
		if (!empty($d['pass']))
			$salt_pass_add = ", pass = '$e_pass', salt = '$e_salt'";

		$sql = "UPDATE users"
			. " SET username = '$e_username'"
			. ", realname = '$e_realname'"
			. $salt_pass_add
			. ", email = '$e_email'"
			. ", is_admin = $e_is_admin"
			. ", disk_quota_mb = $e_disk_quota_mb"
			. ", rights = '$e_rights'"
			. ", session_time = $e_session_time"
			. " WHERE uid = " . $d['uid'];
	}

	$res = rg_sql_query($db, $sql);
	if ($res === FALSE) {
		rg_user_set_error("cannot insert/update user (" . rg_sql_error() . ")");
		return FALSE;
	}
	rg_sql_free_result($res);

	// invalidate cache
	$rg_user_info_cache = array();

	return TRUE;
}

/*
 * Delete a user
 */
function rg_user_remove($db, $uid)
{
	global $rg_user_info_cache;

	$uid = sprintf("%u", $uid);

	$sql = "DELETE FROM users WHERE uid = $uid";
	$res = rg_sql_query($db, $sql);
	if ($res === FALSE) {
		rg_user_set_error("cannot remove user $uid (" . rg_sql_error() . ")");
		return FALSE;
	}
	rg_sql_free_result($res);

	// invalidate cache
	$rg_user_info_cache = array();

	return TRUE;
}

/*
 * Returns info about a user (by uid, user or e-mail)
 */
$rg_user_info_cache = array();
function rg_user_info($db, $uid, $user, $email)
{
	global $rg_user_info_cache;

	$key = $uid . "__" . $user . "__" . $email;
	if (isset($rg_user_info_cache[$key]))
		return $rg_user_info_cache[$key];

	rg_log("user_info: uid/user/email=$uid/$user/$email...");

	$ret = array();
	$ret['ok'] = 0;
	$ret['exists'] = 0;
	$ret['uid'] = 0;
	$ret['is_admin'] = 0;

	// in case we return error
	$rg_user_info_cache[$key] = $ret;

	if ($uid > 0) {
		$add = " AND uid = " . sprintf("%u", $uid);
	} else if (!empty($user)) {
		if (rg_user_ok($user) !== TRUE)
			return FALSE;

		$e_user = rg_sql_escape($db, $user);
		$add = " AND username = '$e_user'";
	} else if (!empty($email)) {
		$e_email = rg_sql_escape($db, $email);
		$add = " AND email = '$e_email'";
	} else {
		return FALSE;
	}

	$sql = "SELECT * FROM users WHERE 1 = 1" . $add;
	$res = rg_sql_query($db, $sql);
	if ($res === FALSE) {
		rg_user_set_error("cannot get info (" . rg_sql_error() . ")");
		return $ret;
	}

	$ret['ok'] = 1;
	$rows = rg_sql_num_rows($res);
	if ($rows > 0)
		$row = rg_sql_fetch_array($res);
	rg_sql_free_result($res);
	if ($rows == 0) {
		rg_user_set_error("user not found");
		return $ret;
	}

	$row['ok'] = 1;
	$row['exists'] = 1;
	rg_log("\tUser found.");

	$rg_user_info_cache[$key] = $row;

	return $row;
}

/*
 * Loads rg_ui based on sid, if possible
 */
function rg_user_login_by_sid($db, $sid, &$rg_ui)
{
	rg_log("user_login_by_sid: sid=$sid...");

	// Make sure it is not passed by client
	$rg_ui = array();
	$rg_ui['uid'] = 0;
	$rg_ui['is_admin'] = 0;
	$rg_ui['username'] = "";

	if (empty($sid)) {
		rg_log("\tNo sid!");
		return FALSE;
	}

	$uid = rg_sess_valid($db, $sid);
	if ($uid == 0)
		return FALSE;

	$rg_ui = rg_user_info($db, $uid, "", "");
	if ($rg_ui['exists'] != 1) {
		rg_log("\tUid $uid does not exists (" . rg_user_error() . ")!");
		rg_user_set_error("invalid uid");
		return FALSE;
	}

	rg_sess_update($db, $sid);

	rg_user_set_last_seen($db, $rg_ui['uid']);

	return TRUE;
}

/*
 * Test if a password is valid
 */
function rg_user_pass_valid($db, $uid, $pass)
{
	rg_log("user_pass_valid: uid=$uid, pass=$pass...");

	if (empty($pass)) {
		rg_user_set_error("password is empty");
		return FALSE;
	}

	$ui = rg_user_info($db, $uid, "", "");
	if ($ui['exists'] != 1) {
		rg_user_set_error("user does not exists");
		return FALSE;
	}

	$sha1pass = rg_user_pass($ui['salt'], $pass);
	if (strcmp($sha1pass, $ui['pass']) != 0) {
		rg_user_set_error("password is not ok");
		return FALSE;
	}

	rg_log("\tPass is valid.");
	return TRUE;
}

/*
 * Auto login the user
 */
function rg_user_auto_login($db, $uid, &$rg_ui)
{
	$rg_ui = rg_user_info($db, $uid, "", "");
	if ($rg_ui['ok'] != 1)
		return FALSE;

	if ($rg_ui['exists'] != 1)
		return FALSE;

	$secure = FALSE;
	if (isset($_SERVER['HTTPS']))
		$secure = TRUE;

	$sid = rg_id(40);
	rg_sess_add($db, $uid, $sid, $rg_ui['session_time']);
	setcookie("sid", $sid, 0, "/", $_SERVER['SERVER_NAME'],
		$secure, TRUE /* httponly */);

	return TRUE;
}

/*
 * Test if login is OK
 */
function rg_user_login_by_user_pass($db, $user, $pass, &$rg_ui)
{
	rg_log("user_login_by_user_pass: user=$user, pass=$pass...");

	$rg_ui = array();
	$rg_ui['uid'] = 0;
	$rg_ui['is_admin'] = 0;

	if (empty($user) || empty($pass)) {
		rg_user_set_error("invalid user or pass");
		return FALSE;
	}

	$rg_ui0 = rg_user_info($db, 0, $user, "");
	if ($rg_ui0['ok'] != 1) {
		rg_user_set_error("internal error");
		return FALSE;
	}
	if ($rg_ui0['exists'] != 1) {
		rg_user_set_error("invalid user or pass");
		return FALSE;
	}

	if ($rg_ui0['suspended'] > 0) {
		rg_user_set_error("invalid user or pass");
		return FALSE;
	}

	if ($rg_ui0['confirmed'] == 0) {
		rg_user_set_error("invalid user or pass");
		return FALSE;
	}

	$sha1pass = rg_user_pass($rg_ui0['salt'], $pass);
	if (strcmp($sha1pass, $rg_ui0['pass']) != 0) {
		rg_user_set_error("invalid user or pass");
		return FALSE;
	}

	$rg_ui = $rg_ui0;
	rg_user_auto_login($db, $rg_ui['uid'], $rg_ui);

	rg_user_set_last_seen($db, $rg_ui['uid']);

	return TRUE;
}

/*
 * Suspend an account
 * 1=suspend, 0=unsuspend
 */
function rg_user_suspend($db, $uid, $op)
{
	rg_log("user_suspend: uid=$uid, op=$op");

	$now = time();

	if ($op == 1)
		$v = $now;
	else
		$v = 0;

	$sql = "UPDATE users SET suspended = $v WHERE uid = $uid";
	$res = rg_sql_query($db, $sql);
	if ($res === FALSE) {
		rg_user_set_error("cannot suspend (" . rg_sql_error() . ")");
		return FALSE;
	}
	rg_sql_free_result($res);

	return TRUE;
}

/*
 * Make/remove admin
 * 1=make, 0=remove
 */
function rg_user_make_admin($db, $uid, $op)
{
	rg_log("user_make_admin: uid=$uid, op=$op");

	$sql = "UPDATE users SET is_admin = $op WHERE uid = $uid";
	$res = rg_sql_query($db, $sql);
	if ($res === FALSE) {
		rg_user_set_error("cannot make admin (" . rg_sql_error() . ")");
		return FALSE;
	}
	rg_sql_free_result($res);

	return TRUE;
}

/*
 * Update last_seen field
 */
function rg_user_set_last_seen($db, $uid)
{
	rg_log("user_set_last_seen: uid=$uid");

	$now = time();

	$sql = "UPDATE users SET last_seen = $now WHERE uid = $uid";
	$res = rg_sql_query($db, $sql);
	if ($res === FALSE) {
		rg_user_set_error("cannot update last seen (" . rg_sql_error() . ")");
		return FALSE;
	}
	rg_sql_free_result($res);

	return TRUE;
}

/*
 * List users
 */
function rg_user_list($db, $url)
{
	rg_log("user_list, url=$url...");

	$ret = "";

	$uid = rg_var_uint("uid");

	$suspend = rg_var_uint("suspend");
	if ($suspend == 1) {
		if (!rg_user_suspend($db, $uid, 1))
			$ret .= "<font color=red>Cannot suspend!</font><br />";
	}

	$unsuspend = rg_var_uint("unsuspend");
	if ($unsuspend == 1) {
		if (!rg_user_suspend($db, $uid, 0))
			$ret .= "<font color=red>Cannot unsuspend!</font><br />";
	}

	$make_admin = rg_var_uint("make_admin");
	if ($make_admin == 1) {
		if (!rg_user_make_admin($db, $uid, 1))
			$ret .= "<font color=red>Cannot make admin!</font><br />";
	}

	$remove_admin = rg_var_uint("remove_admin");
	if ($remove_admin == 1) {
		if (!rg_user_make_admin($db, $uid, 0))
			$ret .= "<font color=red>Cannot remove admin!</font><br />";
	}

	$remove = rg_var_uint("remove");
	if ($remove > 0) {
		if (!rg_user_remove($db, $uid))
			$ret .= "<font color=red>Cannot remove!</font><br />";
	}

	$sql = "SELECT * FROM users ORDER BY username";
	$res = rg_sql_query($db, $sql);
	if ($res === FALSE) {
		rg_user_set_error("cannot get info (" . rg_sql_error() . ")!");
		return FALSE;
	}

	$ret .= "<table>\n";
	$ret .= "<tr>\n";
	$ret .= "	<th>User name</th>\n";
	$ret .= "	<th>Name</th>\n";
	$ret .= "	<th>E-mail</th>\n";
	$ret .= "	<th>Admin?</th>\n";
	$ret .= "	<th>Creation date (UTC)</th>\n";
	$ret .= "	<th>Quota</th>\n";
	$ret .= "	<th>Suspended?</th>\n";
	$ret .= "	<th>Confirmed?</th>\n";
	$ret .= "	<th>Session time</th>\n";
	$ret .= "	<th>Last seen (UTC)</th>\n";
	$ret .= "	<th>Rights</th>\n";
	$ret .= "	<th>Operations</th>\n";
	$ret .= "</tr>\n";
	while (($row = rg_sql_fetch_array($res))) {
		$ret .= "<tr>\n";
		$ret .= "	<td>" . $row['username'] . "</td>\n";
		$ret .= "	<td>" . $row['realname'] . "</td>\n";
		$ret .= "	<td>" . $row['email'] . "</td>\n";
		$ret .= "	<td>" . ($row['is_admin'] == 1 ? "Yes" : "No") . "</td>\n";
		$ret .= "	<td>" . gmdate("Y-m-d", $row['itime']) . "</td>\n";
		if ($row['disk_quota_mb'] > 0)
			$_v = rg_1024($row['disk_quota_mb'] * 1024 * 1024);
		else
			$_v = "unlimited";
		$ret .= "	<td>" . $_v . "</td>\n";
		$ret .= "	<td>" . ($row['suspended'] == 0 ? "No" : "Yes") . "</th>\n";
		$ret .= "	<td>" . ($row['confirmed'] == 0 ? "No" : gmdate("Y-m-d", $row['confirmed'])) . "</th>\n";
		$ret .= "	<td>" . $row['session_time'] . "s</td>\n";
		$v = $row['last_seen'] == 0 ? "-" : gmdate("Y-m-d", $row['last_seen']);
		$ret .= "	<td>" . $v . "</td>\n";
		$v = implode(", ", rg_rights_text("user", $row['rights']));
		$ret .= "	<td>" . $v . "</td>\n";

		// operations
		$_url = $url . "&amp;uid=" . $row['uid'];
		$ret .= "	<td>";

		//	edit
		$ret .= "[<a href=\"$_url&amp;subsubop=3\">Edit</a>]";

		//	suspend
		$v = "suspend=1"; $t = "Suspend";
		if ($row['suspended'] > 0) {
			$t = "Unsuspend";
			$v = "unsuspend=1";
		}
		$ret .= "[<a href=\"$_url&amp;subsubop=1&amp;$v\">$t</a>]";

		//	admin
		$v = "make_admin=1"; $t = "Make admin";
		if ($row['is_admin'] == 1) {
			$t = "Remove admin";
			$v = "remove_admin=1";
		}
		$ret .= "[<a href=\"$_url&amp;subsubop=1&amp;$v\">$t</a>]";

		//	remove
		if ($row['suspended'] > 0)
			$ret .= "[<a href=\"$_url&amp;subsubop=1&amp;remove=1\">Remove!</a>]";

		$ret .= "	</td>";
		$ret .= "</tr>\n";
	}
	$ret .= "</table>\n";
	rg_sql_free_result($res);

	return $ret;
}

/*
 * Returns uid by token, if not expired
 */
function rg_user_forgot_pass_uid($db, $token)
{
	$ret = array();
	$ret['ok'] = 0;
	$ret['uid'] = 0;

	rg_log("user_forgot_pass_uid: token=$token");

	$now = time();
	$e_token = rg_sql_escape($db, $token);

	$sql = "SELECT uid FROM forgot_pass"
		. " WHERE token = '$e_token'"
		. " AND expire > $now";
	$res = rg_sql_query($db, $sql);
	if ($res === FALSE) {
		rg_user_set_error("cannot lookup token (" . rg_sql_error() . ")");
		return $ret;
	}

	$ret['ok'] = 1;

	$rows = rg_sql_num_rows($res);
	if ($rows > 0)
		$row = rg_sql_fetch_array($res);
	rg_sql_free_result($res);
	if ($rows == 0)
		return $ret;

	$ret['uid'] = $row['uid'];

	return $ret;
}

/*
 * Reset password function (send mail) - helper
 */
function rg_user_forgot_pass_mail_prepare($db, $email)
{
	rg_log("user_forgot_pass_mail_prepare: email=$email");

	$ret = array();
	$ret['ok'] = 0;
	$ret['exists'] = 0;

	$expire = time() + 24 * 3600;
	$token = rg_id(40);

	$r = rg_user_info($db, 0, "", $email);
	if ($r['ok'] == 0) {
		rg_log("\tInternal error.");
		return $ret;
	}
	if ($r['exists'] == 0) {
		rg_log("\tUser does not exists.");
		$ret['ok'] = 1;
		return $ret;
	}
	$uid = $r['uid'];

	// store token in database
	$sql = "INSERT INTO forgot_pass (token, uid, expire)"
		. " VALUES ('$token', $uid, $expire)";
	$res = rg_sql_query($db, $sql);
	if ($res === FALSE) {
		rg_user_set_error("cannot query (" . rg_sql_error() . ")");
		return $ret;
	}
	rg_sql_free_result($res);

	$ret['ok'] = 1;
	$ret['exists'] = 1;
	$ret['token'] = $token;

	rg_log("DEBUG: user_forgot_pass_mail_prepare: ret=" . rg_array2string($ret));

	return $ret;
}

/*
 * Reset password function (send mail)
 */
function rg_user_forgot_pass_mail($db, $email)
{
	global $rg_admin_name, $rg_admin_email;

	rg_log("user_forgot_pass_mail: email=$email");

	$ret = array();
	$ret['ok'] = 0;
	$ret['exists'] = 0;

	$r = rg_user_forgot_pass_mail_prepare($db, $email);
	if ($r['exists'] != 1) {
		rg_log("\tUser does not exists.");
		return $r;
	}

	$ret['exists'] = 1;

	$headers = "From: $rg_admin_name <$rg_admin_email>";

	if (isset($_SERVER['HTTPS']))
		$proto = "https";
	else
		$proto = "http";

	if (!mail($email,
		"Forgot password",
		"Hello!\n\n"
		. "If you want to reset the password, follow:\n"
		. $proto . "://"
		. @$_SERVER['HTTP_HOST']
		. rg_re_url("/op/forgot_link") . "&forgot_token=" . $r['token']
		. "\n\nRocketGit team",
		$headers,
		"-f $rg_admin_email")) {
		rg_user_set_error("cannot send mail ($php_errormsg)!");
		return $ret;
	}

	$ret['ok'] = 1;

	rg_log("DEBUG: user_forgot_pass_mail: ret=" . rg_array2string($ret) . ".");
	return $ret;
}

/*
 * After reseting the pass, we have to destroy all 'reset pass' requests
 */
function rg_user_forgot_pass_destroy($db, $uid)
{
	rg_log("user_forgot_pass_destroy: uid=$uid");

	$sql = "DELETE FROM forgot_pass WHERE uid = $uid";
	$res = rg_sql_query($db, $sql);
	if ($res === FALSE) {
		rg_user_set_error("cannot query (" . rg_sql_error() . ")");
		return FALSE;
	}
	rg_sql_free_result($res);

	return TRUE;
}

function rg_user_set_pass($db, $uid, $pass)
{
	rg_log("user_set_pass: uid=$uid pass=$pass");

	$e_salt = rg_id(40);
	$e_sha1pass = rg_user_pass($e_salt, $pass);

	$sql = "UPDATE users SET"
		." salt = '$e_salt'"
		. ", pass = '$e_sha1pass'"
		. " WHERE uid = " . $uid;
	$res = rg_sql_query($db, $sql);
	if ($res === FALSE) {
		rg_user_set_error("cannot update pass (" . rg_sql_error() . ")");
		return FALSE;
	}
	rg_sql_free_result($res);

	return TRUE;
}

/*
 * Confirm account creation (send mail)
 */
function rg_user_confirm_send($email, $token)
{
	global $rg_admin_name, $rg_admin_email;

	rg_log("user_confirm_send: email=$email, token=$token");

	$headers = "From: $rg_admin_name <$rg_admin_email>";

	$proto = "http://";
	if (isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'], "on") == 0))
		$proto = "https://";

	if (!mail($email,
		"Account creation confirmation",
		"Hello!\n\n"
		. "Please confirm your account creation following:\n"
		. $proto
		. @$_SERVER['HTTP_HOST']
		. rg_re_url("/op/confirm") . "&token=" . $token
		. "\n\nRocketGit team",
		$headers,
		"-f $rg_admin_email")) {
		rg_user_set_error("cannot send mail ($php_errormsg)!");
		return FALSE;
	}

	return TRUE;
}

/*
 * Confirm account creation
 */
function rg_user_confirm($db, $token)
{
	$now = time();

	$sql = "SELECT uid FROM users WHERE confirm_token = '$token'";
	$res = rg_sql_query($db, $sql);
	if ($res === FALSE) {
		rg_user_set_error("cannot search for token (" . rg_sql_error() . ")");
		return FALSE;
	}
	$rows = rg_sql_num_rows($res);
	if ($rows > 0)
		$row = rg_sql_fetch_array($res);
	rg_sql_free_result($res);
	if ($rows == 0) {
		rg_user_set_error("cannot find token (" . rg_sql_error() . ")");
		return FALSE;
	}
	$uid = $row['uid'];

	$sql = "UPDATE users SET confirmed = $now"
		. " WHERE uid = $uid";
	$res = rg_sql_query($db, $sql);
	if ($res === FALSE) {
		rg_user_set_error("cannot update confirmed (" . rg_sql_error() . ")");
		return FALSE;
	}
	rg_sql_free_result($res);

	return $uid;
}

/*
 * Add a suggestion to database
 */
function rg_user_suggestion($db, $uid, $email, $suggestion)
{
	$e_email = rg_sql_escape($db, $email);
	$e_suggestion = rg_sql_escape($db, $suggestion);

	$sql = "INSERT INTO suggestions (uid, email, suggestion)"
		. " VALUES ($uid, '$e_email', '$e_suggestion')";
	$res = rg_sql_query($db, $sql);
	if ($res === FALSE) {
		rg_user_set_error("cannot add suggestion (" . rg_sql_error() . ")");
		return FALSE;
	}
	rg_sql_free_result($res);

	return TRUE;
}

?>
