<?php
rg_log("/inc/user/repo-page");

$_repo_page = "";

if (rg_user_ok($rr['user']) !== TRUE) {
	$_repo_page .= "Invalid user!";
	return;
}

if (rg_repo_ok($rr['repo']) !== TRUE) {
	$_repo_page .= "Invalid repo!";
	return;
}

// Git dir is done asynchronous, loop till is OK
while (1) {
	$ri = rg_repo_info($db, $rr);
	if ($ri === FALSE) {
		$_repo_page .= "Internal error!";
		return;
	}

	if (($ri['ok'] != 1) || ($ri['exists'] != 1) || ($ri['deleted'] == 1)) {
		$_repo_page .= "Invalid repository!";
		return;
	}

	if ($ri['git_dir_done'] == 1)
		break;

	rg_log("git dir is not ready yet. Sleep 1 second and loop.");
	sleep(1);
	rg_repo_invalidate_cache($rr);
}

$ui = rg_user_info($db, 0, $rr['user'], "");
if ($ui === FALSE) {
	$_repo_page .= "Invalid user!";
	return;
}
// TODO: should we test against $ri?!
$can_admin = (rg_repo_allow($db, $ri, $rg_ui, "A") === TRUE) ? 1 : 0;

$_more = array(
	"owner" => $rr['user'],
	"url_user" => rg_re_userpage($ui),
	"url_repo" => rg_re_repopage($ui, $rr['repo']),
	"ssh" => rg_re_repo_ssh($rr),
	"git" => rg_re_repo_git($rr),
	"can_admin" => $can_admin
	);
$_more = array_merge($_more, $ri);
$_more['default_rights'] = implode(", ", rg_rights_text("repo", $ri['default_rights']));
$_more['max_commit_size'] = $ri['max_commit_size'] == 0 ? "unlimited" : rg_1024($ri['max_commit_size']);
$_more['max_users'] = $ri['max_users'] == 0 ? "unlimited" : rg_1000($ri['max_users']);
$_more['hints'] = "";

$repo_dir = rg_repo_name2base($rr) . $rr['repo'] . ".git";
rg_log("repo_dir=$repo_dir");
putenv("GIT_DIR=$repo_dir");

// default is "source" tab
if (empty($subop))
	$subop = "source";

$_more['repo_body'] = "";
$_more['repo_right'] = "";
$_more['branches_and_tags'] = "";
$_repo_body = "";
$_repo_right = "";

// build urls list
$urls = array();
if ($rg_ssh_port != 0)
	$urls[]['HTML:url'] = '<a href="' . $_more['ssh'] . '">' . $_more['ssh'] . '</a>';
if ($rg_git_port != 0)
	$urls[]['HTML:url'] = '<a href="' . $_more['git'] . '">' . $_more['git'] . '</a>';
$_more['HTML:urls'] = rg_template_table("repo/urls", $urls, $_more);

if (strcmp($subop, "admin") == 0) {
	if ($can_admin != 1)
		rg_security_violation("User is not admin!");

	include($INC . "/user/repo/admin.php");
	$_repo_body .= $_admin;
} else if (strcmp($subop, "source") == 0) {
	$subop = array_shift($paras);

	$type_ref = rg_git_parse_ref($paras);
	$ref = $type_ref['ref_path'];
	$_more = array_merge($_more, $type_ref);

	$bt = rg_git_branches_and_tags($repo_dir, $_more['url_repo'],
		$type_ref['ref_url']);
	rg_log("DEBUG: _more: " . rg_array2string($_more));
	$_more = array_merge($_more, $bt);

	$hints = array();
	if ($rg_ssh_port != 0)
		$hints[]['HTML:hint'] = rg_template("hints/repo/clone_ssh.html", $_more);
	if ($rg_git_port != 0)
		$hints[]['HTML:hint'] = rg_template("hints/repo/clone_git.html", $_more);
	if (rg_rights_allow($ri['default_rights'], "H")) // anonymous push?
		$hints[]['HTML:hint'] = rg_template("hints/repo/anon_push.html", $_more);
	rg_log("DEBUG: hints1: " . rg_array2string($hints));
	$_more['HTML:hints'] = rg_template_table("hints/list", $hints, $_more);
	rg_log("DEBUG: hints: " . $_more['HTML:hints']);

	$_repo_body .= rg_template("repo/source.html", $_more);

	rg_log("DEBUG: subop=[$subop]");
	if (strcmp($subop, "tree") == 0) {
		$type = array_shift($paras);
		rg_log("DEBUG: tree: type=$type");
		if (strcmp($type, "blob") == 0) {
			// find hash of blob
			$_path = implode("/", $paras);
			$_more['path'] = "/" . $_path;
			$_tree = rg_git_ls_tree($ref, $_path);
			if ($_tree === FALSE) {
				$_repo_body .= "Invalid path!";
			} else {
				$blob = $_tree[0];
				$_hash = $_tree[0]['ref'];
				$c = rg_git_content($_hash);
				$_content = array(
					"hash" => $_hash,
					"HTML:content" => rg_template_list($c)
					);
				$blob = array_merge($blob, $_content);
				$_repo_body .= rg_template("repo/blob.html",
					array_merge($blob, $_more));
			}
		} else if (strcmp($type, "tree") == 0) {
			// find treeish of dir
			$_path = implode("/", $paras);
			$_more['path'] = "/" . $_path;
			$_tree = rg_git_ls_tree($ref, $_path);
			if ($_tree === FALSE) {
				$_repo_body .= "Invalid path!";
			} else {
				$_hash = $_tree[0]['ref'];
				$_tree = rg_git_ls_tree($_hash, "");
				$_repo_body .= rg_template_table("repo/tree", $_tree, $_more);
			}
		} else { // default is to show root tree
			$_more['path'] = "";
			$_tree = rg_git_ls_tree($ref, "");
			$_repo_body .= rg_template_table("repo/tree", $_tree, $_more);
		}
	} else { // log is default
		// show the log
		$log = rg_git_log(10, "", $ref, FALSE);
		if ($log === FALSE) {
			rg_log("Error generating log (" . rg_git_error() . ")!");
			$_repo_body .= "Error generating log!";
			// TODO: rg_internal_error?
		} else if (empty($log)) {
			$_repo_body .= rg_template("repo/not_init.html", $_more);
		} else {
			//rg_log("DEBUG: log: " . print_r($log, TRUE));
			$_repo_body .= rg_git_log_template($log,
				"repo/log", $_more);
		}

		$type = array_shift($paras);
		rg_log("DEBUG: log: type=$type");
		if (strcmp($type, "commit") == 0) {
			$commit = rg_git_reference($paras[0]);
			rg_log("Show diff for commit [$commit]");

			if (strstr($commit, "..")) {
				$t = explode("..", $commit);
				$first = $t[0];
				$second = $t[1];
			} else {
				$first = $commit . "~1";
				$second = $commit;
			}

			$log = rg_git_log(1, $first, $second, TRUE);
			rg_log("log: " . rg_array2string($log));

			// stats
			$_repo_body .= rg_git_files_stats($log[0]['files'], "repo/fstat");

			// diff
			//rg_log("patch: " . print_r($log[0]['patches'], TRUE));
			$_repo_body .= rg_git_diff($log[0]['patches'], "repo/diff.html");
		}
	}
	$_repo_right = rg_template("repo/source_r.html", $_more);
} else if (strcmp($subop, "bug") == 0) {
	include($INC . "/user/repo/bug/main.php");
	$_repo_body .= rg_template("repo/bug/main.html", $_more);
} else if (strcmp($subop, "stats") == 0) {
	$_repo_body .= rg_template("repo/stats.html", $_more);
} else if (strcmp($subop, "mr") == 0) {
	if (empty($paras[0])) {
		$_repo_body .= rg_template("repo/mrs.html", $_more);

		$r = rg_mr_load($db, $ri['repo_id'], 20);
		if ($r === FALSE) {
			$_repo_body .= "Error getting merge request list ("
				. rg_mr_error() . ").";
		} else {
			$_repo_body .= rg_template_table("repo/mr/list", $r, $_more);
		}
	} else {
		$mr = preg_replace('/[^0-9a-zA-Z_]/', '', $paras[0]);
		do {
			$mri = rg_mr_load_one($db, $ri['repo_id'], $mr);
			if ($mri === FALSE) {
				$_repo_body .= "Error getting merge request (" . rg_mr_error() . ").";
				break;
			}

			$_log = rg_git_log(0, $mri['old_rev'], $mri['new_rev'], TRUE);
			if ($_log === FALSE) {
				$_repo_body .= "Error generating patch (" . rg_git_error() . ").";
				break;
			}

			$mri['HTML:diff'] = rg_git_log_template($_log, "repo/log", $_more);
			$_repo_body .= rg_template("repo/mr/page.html", $mri, $_more);
		} while (0);
	}
}

$_more['HTML:repo_body'] = $_repo_body;
$_more['HTML:repo_right'] = $_repo_right;
$_repo_page = rg_template("repo/main.html", $_more);
?>
